aboutsummaryrefslogtreecommitdiff
path: root/arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h
diff options
context:
space:
mode:
authorMichalis Spyrou <michalis.spyrou@arm.com>2017-11-23 09:49:51 +0000
committerAnthony Barbier <anthony.barbier@arm.com>2018-11-02 16:42:33 +0000
commit780db4eb6a9e3dee565d14f36d772038cd3253da (patch)
tree53490d6a03bdeb26d77bc8840d1dbf6027e81f5c /arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h
parentd7ba5397b676c966cb5069c7187a12a0c35305f5 (diff)
downloadComputeLibrary-780db4eb6a9e3dee565d14f36d772038cd3253da.tar.gz
COMPMID-471 Implement Deconvolution on OpenCL
Change-Id: Ie00c6b08a51d30c5ce2637d40ee3d165b8a68686 Reviewed-on: https://eu-gerrit-1.euhpc.arm.com/110311 Reviewed-by: Pablo Tello <pablo.tello@arm.com> Reviewed-by: Georgios Pinitas <georgios.pinitas@arm.com> Tested-by: Jenkins <bsgcomp@arm.com>
Diffstat (limited to 'arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h')
-rw-r--r--arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h61
1 files changed, 35 insertions, 26 deletions
diff --git a/arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h b/arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h
index 8757bc63aa..091a928db6 100644
--- a/arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h
+++ b/arm_compute/runtime/NEON/functions/NEDeconvolutionLayer.h
@@ -1,5 +1,5 @@
/*
- * Copyright (c) 2017 ARM Limited.
+ * Copyright (c) 2017, 2018 ARM Limited.
*
* SPDX-License-Identifier: MIT
*
@@ -24,7 +24,6 @@
#ifndef __ARM_COMPUTE_NEDECONVOLUTIONLAYER_H__
#define __ARM_COMPUTE_NEDECONVOLUTIONLAYER_H__
-#include "arm_compute/runtime/NEON/functions/NEDeconvolutionLayerUpsample.h"
#include "arm_compute/runtime/NEON/functions/NEDirectConvolutionLayer.h"
#include "arm_compute/core/Types.h"
@@ -39,13 +38,13 @@ namespace arm_compute
{
/** Function to run the deconvolution layer.
*
- * The operation is similar to convolution but it's implemented by up-sampling the inputs with zeros insertions between the inputs and convolving
- * the kernels on the up-sampled result.
+ * Deconvolution Layer is the backward pass of Convolution Layer. First we transform the input depending on the stride and pad info and then perfrom a 1x1
+ * convolution pass. Input stride defines how many zeroes we should put between each element of the input, pad is the amount of padding and finaly a is a user
+ * specified value where a < stride - 1 that increases the padding top and right of the input image.
*
- * Before the Deconvolution is done, up-scaling the first 2D with zeros is performed. The relation between input to
- * output is as follows:
- * width_output = round((width_input − 1) ∗ upscale_x − 2 ∗ padding_x + kernel_x + a_x )
- * height_output = round((height_input − 1) ∗ upscale_y − 2 ∗ padding_y + kernel_y + a_y )
+ * The relation between input to output is as follows:
+ * width_output = round((width_input − 1) ∗ (stride_x - 1) − 2 ∗ padding_x + kernel_x + inner_border_right )
+ * height_output = round((height_input − 1) ∗ (stride_y - 1) − 2 ∗ padding_y + kernel_y + inner_border_top )
*
* where
* width is the size of the first input dimension.
@@ -53,44 +52,54 @@ namespace arm_compute
* width_output is the size of the first output dimension.
* height_output is the size of the second output dimension.
* kernel_x and kernel_y are the convolution sizes in x and y.
- * ax and ay the number of zeros added to the top and right edges of the input.
- * upscale_x and upscale_y how much to scale the X and Y axis.
+ * inner_border_right and inner_border_top the number of zeros added to the top and right edges of the input.
+ * stride_x and stride_y is the input stride of the first and second dimension.
*
* This function calls the following NEON kernels:
*
- * -# @ref NEDeconvolutionLayerUpsampleKernel
* -# @ref NEDirectConvolutionLayer
*
*/
class NEDeconvolutionLayer : public IFunction
{
public:
- /** Constructor */
+ /** Default constructor */
NEDeconvolutionLayer(std::shared_ptr<IMemoryManager> memory_manager = nullptr);
+
+ /** Prevent instances of this class from being copied (As this class contains pointers) */
+ NEDeconvolutionLayer(const NEDeconvolutionLayer &) = delete;
+ /** Prevent instances of this class from being copied (As this class contains pointers) */
+ NEDeconvolutionLayer &operator=(const NEDeconvolutionLayer &) = delete;
+ /** Allow instances of this class to be moved */
+ NEDeconvolutionLayer(NEDeconvolutionLayer &&) = default;
+ /** Allow instances of this class to be moved */
+ NEDeconvolutionLayer &operator=(NEDeconvolutionLayer &&) = default;
+ /** Default destructor */
+ virtual ~NEDeconvolutionLayer() = default;
/** Set the input, weights, biases and output tensors.
*
* @param[in,out] input Input tensor. 3 lower dimensions represent a single input, and an optional 4th dimension for batch of inputs. Data types supported: F32.
* @param[in] weights The 4d weights with dimensions [width, height, OFM, IFM]. Data type supported: Same as @p input.
* @param[in] bias Optional, ignored if NULL. The biases have one dimension. Data type supported: Same as @p input.
* @param[out] output Output tensor. The output has the same number of dimensions as the @p input.
* @param[in] info Contains padding and policies to be used in the deconvolution, this is decribed in @ref PadStrideInfo.
* @param[in] ax The number of zeros added to right edge of the input.
* @param[in] ay The number of zeros added to top edge of the input.
* @param[in] upscalex How much to scale the X axis.
* @param[in] upscaley How much to scale the Y axis.
*
*/
void configure(ITensor *input, const ITensor *weights, const ITensor *bias, ITensor *output, const PadStrideInfo &info,
- unsigned int ax, unsigned int ay, float upscalex, float upscaley);
+ unsigned int inner_border_right, unsigned int inner_border_top);
// Inherited methods overridden:
void run() override;
private:
- MemoryGroup _memory_group;
- NEDeconvolutionLayerUpsample _scale_f;
- NEDirectConvolutionLayer _conv_f;
- Tensor _scaled_output;
+ MemoryGroup _memory_group;
+ NEDirectConvolutionLayer _conv_f;
+ Tensor _scaled_output;
+ ITensor *_input;
+ PadStrideInfo _info;
+ std::pair<unsigned int, unsigned int> _inner_border;
};
} // arm_compute
#endif /* __ARM_COMPUTE_NEDECONVOLUTIONLAYER_H__ */
@@ -53,44 +52,54 @@ namespace arm_compute
/** Set the input, weights, biases and output tensors.
*
- * @param[in,out] input Input tensor. 3 lower dimensions represent a single input, and an optional 4th dimension for batch of inputs. Data types supported: F32.
- * @param[in] weights The 4d weights with dimensions [width, height, OFM, IFM]. Data type supported: Same as @p input.
- * @param[in] bias Optional, ignored if NULL. The biases have one dimension. Data type supported: Same as @p input.
- * @param[out] output Output tensor. The output has the same number of dimensions as the @p input.
- * @param[in] info Contains padding and policies to be used in the deconvolution, this is decribed in @ref PadStrideInfo.
- * @param[in] ax The number of zeros added to right edge of the input.
- * @param[in] ay The number of zeros added to top edge of the input.
- * @param[in] upscalex How much to scale the X axis.
- * @param[in] upscaley How much to scale the Y axis.
+ * @param[in,out] input Input tensor. 3 lower dimensions represent a single input, and an optional 4th dimension for batch of inputs. Data types supported: F32.
+ * @param[in] weights The 4d weights with dimensions [width, height, OFM, IFM]. Data type supported: Same as @p input.
+ * @param[in] bias Optional, ignored if NULL. The biases have one dimension. Data type supported: Same as @p input.
+ * @param[out] output Output tensor. The output has the same number of dimensions as the @p input.
+ * @param[in] info Contains padding and policies to be used in the deconvolution, this is decribed in @ref PadStrideInfo.
+ * @param[in] inner_border_right The number of zeros added to right edge of the input.
+ * @param[in] inner_border_top The number of zeros added to top edge of the input.
*
*/